/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*******************************************************************************
 *  MSP432 I2C - EUSCI_B0_BASE I2C TX multiple bytes to Secondary
 *
 *  Description: This demo uses I2C of the MSP to program the
 *
 *                                /|\  /|\
 *                MSP432P4111      10k  10k
 *                   slave         |    |
 *             -----------------   |    |
 *            |     P1.6/UCB0SDA|<-|----+->
 *            |                 |  |
 *            |                 |  |
 *            |     P1.7/UCB0SCL|<-+------>
 *            |                 |
 *
 *
 *
 ******************************************************************************/
/* DriverLib Includes */
#include <ti/devices/msp432p4xx/driverlib/driverlib.h>

#include <tps6594BinSource.h>

#define STARTINGPAGE0_ADDRESS   0x48

/* Register Addresses */
#define DEVICE_ID               0x0001
#define NVM_ID                  0x0002
#define NVM_REV                 0x0003
#define CRC_1                   0x00F0
#define CRC_2                   0x00F1
#define USER_EE_CTRL_1          0x00A2
#define USER_EE_CTRL_2          0x00A3
#define MASK_INCLUDE            0
#define MASK_EXCLUDE            1
#define PAGE1_I2C1_OFFSET       34
#define PAGE1_I2C2_OFFSET       35
#define ENDRV_STAT_OFFSET       3
#define ADDRESS                 0
#define DATA                    1
#define REG_CRC_EN_OFFSET       0x0018

uint8_t page0_I2C_addr;

const uint8_t userDataCRC_mask[256][2] =
{
    {0x00, 0x00},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xBF},
    {0xFF, 0x3F},
    {0xFF, 0xBB},
    {0xFF, 0x3F},
    {0xFF, 0xBF},
    {0xFF, 0x3F},
    {0xFF, 0xBB},
    {0xFF, 0x3F},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0x3F},
    {0xFF, 0x3F},
    {0xFF, 0x3F},
    {0xFF, 0x3F},
    {0xFF, 0xF3},
    {0xFF, 0xF3},
    {0xFF, 0xF3},
    {0xFF, 0xF3},
    {0xFF, 0xFF},
    {0x00, 0x00},
    {0xFF, 0x01},
    {0xFF, 0xFE},
    {0xFF, 0xFE},
    {0xFF, 0xFE},
    {0xFF, 0x7F},
    {0xFF, 0x3F},
    {0xFF, 0x3F},
    {0xFF, 0x3F},
    {0xFF, 0x3F},
    {0xFF, 0x21},
    {0xFF, 0x7F},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0x10},
    {0xFF, 0x10},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0x11},
    {0xFF, 0xFF},
    {0xFF, 0x07},
    {0x00, 0x00},
    {0x00, 0x00},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0x0F},
    {0xFF, 0xFF},
    {0xFF, 0x03},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0x3F},
    {0xFF, 0xBB},
    {0xFF, 0xBB},
    {0xFF, 0x0B},
    {0xFF, 0xBB},
    {0xFF, 0xBB},
    {0xFF, 0x03},
    {0xFF, 0xF3},
    {0xFF, 0xF3},
    {0xFF, 0x3F},
    {0xFF, 0x20},
    {0xFF, 0x0B},
    {0xFF, 0xD6},
    {0x00, 0x00},
    {0xFF, 0x0F},
    {0xFF, 0xAB},
    {0xFF, 0x09},
    {0xFF, 0x3F},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0xFF, 0xFF},
    {0xFF, 0x03},
    {0xFF, 0xFF},
    {0xFF, 0xF9},
    {0xFF, 0x03},
    {0xFF, 0xFB},
    {0x7F, 0x0F},
    {0x00, 0x00},
    {0xFF, 0x01},
    {0xFF, 0xFF},
    {0xF8, 0x18},
    {0x00, 0x00},
    {0xEF, 0x0F},
    {0xF0, 0xF0},
    {0xFF, 0x03},
    {0xFF, 0x1F},
    {0xFF, 0x07},
    {0x00, 0x00},
    {0xFF, 0x1F},
    {0xFF, 0x1F},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0x0F},
    {0xFF, 0x01},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xEF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0x00, 0x00},
    {0xFF, 0x01},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xEF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0xFF, 0xFF},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0xFF, 0xFF},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0xFF, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0xFF, 0x00},
    {0xFF, 0x00},
    {0xFF, 0x00},
    {0xFF, 0x00},
    {0xFF, 0x00},
    {0xFF, 0x00},
    {0xFF, 0x00},
    {0xFF, 0x00},
    {0xFF, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0xFF, 0xFF},  // 0x401
    {0x00, 0x00},
    {0xFF, 0x7F},  // 0x403
    {0xFF, 0x7F},  // 0x404
    {0xFF, 0xFF},  // 0x405
    {0x00, 0x00},
    {0xFF, 0xFF},  // 0x407
    {0x00, 0x00},
    {0xFF, 0xFF},  // 0x409
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00},
    {0x00, 0x00}
};

const uint8_t userDataNonNVM_data[255][2] =
{
    {0x0022, 0x00},
    {0x0080, 0x00},
    {0x0081, 0x00},
    {0x0082, 0x08},  // ENABLE_DRV_STAT
    {0x0085, 0x00},
    {0x0086, 0x00},
    {0x008F, 0x00},
    {0x0090, 0x00},
    {0x0091, 0x00},
    {0x0092, 0x00},
    {0x0093, 0x00},
    {0x0094, 0x00},
    {0x0096, 0x00},
    {0x0098, 0x00},
    {0x0099, 0x00},
    {0x009A, 0x00},
    {0x009B, 0x00},
    {0x009C, 0x00},
    {0x009D, 0x00},
    {0x009E, 0x00},
    {0x009F, 0x00},
    {0x00AB, 0x00},
    {0x00C9, 0x00},
    {0x00CA, 0x00},
    {0x00CB, 0x00},
    {0x00CC, 0x00},
    {0x00F0, 0x00},  // 0x401 is to 0x0F0 for CRC
    {0x00F2, 0x7F},  // 0x403 is to 0x0F2
    {0x00F3, 0x7F},  // 0x404 is to 0x0F3
    {0x00F6, 0x0A}   // 0x407 is to 0x0F6
};


//![Simple I2C Config]
/* I2C Master Configuration Parameter */
const eUSCI_I2C_MasterConfig i2cConfig =
{
        EUSCI_B_I2C_CLOCKSOURCE_SMCLK,          // SMCLK Clock Source
        3000000,                                // SMCLK = 3MHz
        EUSCI_B_I2C_SET_DATA_RATE_400KBPS,      // Desired I2C Clock of 400khz
        0,                                      // No byte counter threshold
        EUSCI_B_I2C_NO_AUTO_STOP                // No Autostop
};
//![Simple I2C Config]

void writeData(uint8_t i2cAddress, uint8_t registerAddress, uint8_t data)
{

    MAP_I2C_setSlaveAddress(EUSCI_B0_BASE, i2cAddress);

    /* Making sure the last transaction has been completely sent out */
    while (MAP_I2C_masterIsStopSent(EUSCI_B0_BASE) == EUSCI_B_I2C_SENDING_STOP);

    /* Sending the initial start condition */
    MAP_I2C_masterSendMultiByteStart(EUSCI_B0_BASE, (uint8_t)registerAddress);
    MAP_I2C_masterSendMultiByteNext(EUSCI_B0_BASE, data);
    MAP_I2C_masterSendMultiByteStop(EUSCI_B0_BASE);

    MAP_Interrupt_enableSleepOnIsrExit();

    MAP_I2C_enableInterrupt(EUSCI_B0_BASE, EUSCI_B_I2C_STOP_INTERRUPT);
    MAP_PCM_gotoLPM0InterruptSafe();
}

uint8_t readData(uint8_t i2cAddress, uint8_t registerAddress)
{
    uint8_t RXData;

    MAP_I2C_setSlaveAddress(EUSCI_B0_BASE, i2cAddress);

    /* Making sure the last transaction has been completely sent out */
    while(MAP_I2C_masterIsStopSent(EUSCI_B0_BASE) == EUSCI_B_I2C_SENDING_STOP);

    /* Sending the initial start condition */
    MAP_I2C_masterSendMultiByteStart(EUSCI_B0_BASE, (uint8_t)registerAddress);

    // Send the restart condition, read one byte, send the stop condition right away
    EUSCI_B0->CTLW0 &= ~(EUSCI_B_CTLW0_TR);
    EUSCI_B0->CTLW0 |= EUSCI_B_CTLW0_TXSTT;
    while(MAP_I2C_masterIsStartSent(EUSCI_B0_BASE));
    EUSCI_B0->CTLW0 |= EUSCI_B_CTLW0_TXSTP;
    while(!(EUSCI_B0->IFG & EUSCI_B_IFG_RXIFG0));
    RXData = EUSCI_B0->RXBUF;

    MAP_Interrupt_enableSleepOnIsrExit();
    // re-enable RX interrupt
    //MAP_I2C_enableInterrupt(EUSCI_B0_BASE, EUSCI_B_I2C_RECEIVE_INTERRUPT0);
    MAP_I2C_enableInterrupt(EUSCI_B0_BASE, EUSCI_B_I2C_STOP_INTERRUPT);
    MAP_PCM_gotoLPM0InterruptSafe();

    return RXData;
}

uint16_t crcD8(uint16_t currentCRC, uint8_t data)
{
    uint8_t ii;
    uint16_t scratchData, newCrc, currData;

    newCrc = currentCRC;
    scratchData = (uint16_t)data;
    scratchData = scratchData << 8;
    for(ii=0;ii<8;ii++)
    {
        currData = (scratchData & 0x8000) ^ (newCrc & 0x8000); // XOR MSBs
        newCrc= newCrc << 1;  // rotate crc left by 1
        if(currData)  // result of XOR MSBs was one
        {
            newCrc ^= 0x755A;  // XOR bits which were previously in locations 0,2,3,5,7,9,11,12,13 (1,3,4,6,8,10,12,13,14-> 0x755A)
            newCrc |= 0x0001;  // Bit 1 position is the result of the XOR MSBs
        }
        scratchData = scratchData << 1;  // rotate data by 1
    }
    return newCrc;
}

int main(void)
{
    volatile uint32_t ii;
    volatile uint16_t includeCRC, excludeCRC;
    volatile uint8_t tempByte, NonNVMIndex, NVMIndex;

    /* Disabling the Watchdog */
    MAP_WDT_A_holdTimer();

    /* Select Port 1 for I2C - Set Pin 6, 7 to input Primary Module Function,
     *   (UCB0SIMO/UCB0SDA, UCB0SOMI/UCB0SCL).
     */
    MAP_GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P1,
            GPIO_PIN6 + GPIO_PIN7, GPIO_PRIMARY_MODULE_FUNCTION);

    //![Simple I2C Example]
    /* Initializing I2C Master to SMCLK at 400kbs with no autostop */
    MAP_I2C_initMaster(EUSCI_B0_BASE, &i2cConfig);

    /* Specify slave address */
    //MAP_I2C_setSlaveAddress(EUSCI_B0_BASE, SLAVE_ADDRESS);

    /* Set Master in transmit mode */
    MAP_I2C_setMode(EUSCI_B0_BASE, EUSCI_B_I2C_TRANSMIT_MODE);

    /* Enable I2C Module to start operations */
    MAP_I2C_enableModule(EUSCI_B0_BASE);
    
    /* Enable and clear the interrupt flag */
    MAP_I2C_clearInterruptFlag(EUSCI_B0_BASE,
            EUSCI_B_I2C_TRANSMIT_INTERRUPT0 + EUSCI_B_I2C_NAK_INTERRUPT);
    
    /* Enable master transmit interrupt */
    MAP_I2C_enableInterrupt(EUSCI_B0_BASE,
            EUSCI_B_I2C_TRANSMIT_INTERRUPT0 + EUSCI_B_I2C_NAK_INTERRUPT);
    MAP_Interrupt_enableInterrupt(INT_EUSCIB0);
    
    /* Configuring P1.1 as an input and enabling interrupts */
    MAP_GPIO_setAsInputPinWithPullUpResistor(GPIO_PORT_P1, GPIO_PIN1);
    MAP_GPIO_clearInterruptFlag(GPIO_PORT_P1, GPIO_PIN1);
    MAP_GPIO_enableInterrupt(GPIO_PORT_P1, GPIO_PIN1);
    MAP_Interrupt_enableInterrupt(INT_PORT1);

    MAP_Interrupt_enableMaster();

    // wait for transaction to occur
    MAP_Interrupt_enableSleepOnIsrExit();
    MAP_PCM_gotoLPM0InterruptSafe();

    page0_I2C_addr = STARTINGPAGE0_ADDRESS;

    // Unlock NVM
    writeData(page0_I2C_addr,USER_EE_CTRL_1,0x00);
    writeData(page0_I2C_addr,USER_EE_CTRL_1,0x98);
    writeData(page0_I2C_addr,USER_EE_CTRL_1,0xB8);
    writeData(page0_I2C_addr,USER_EE_CTRL_1,0x13);
    writeData(page0_I2C_addr,USER_EE_CTRL_1,0x7D);

    tempByte = readData(page0_I2C_addr,USER_EE_CTRL_2);
    if(!((tempByte & 0xC0) == 0xC0))
    {
        while(1);  // NVM locked
    }

    // Disable PFSM
    writeData(page0_I2C_addr,USER_EE_CTRL_2,0xC1);

    /*
     * Update device address to value from BinSource.
     *
     */
    writeData(page0_I2C_addr+1,(uint8_t)page1Image[PAGE1_I2C1_OFFSET].address,page1Image[PAGE1_I2C1_OFFSET].data);
    /*
     * New I2C address to be used through remainded of programming
     */
    page0_I2C_addr = page1Image[PAGE1_I2C1_OFFSET].data;
    writeData(page0_I2C_addr+1,(uint8_t)page1Image[35].address,page1Image[35].data);

    /*
     * Unlock Frequency selection
     *
     */
    tempByte = readData(page0_I2C_addr+1,0x18);
    tempByte |= 0x08;
    writeData(page0_I2C_addr+1,0x18,tempByte);

/*******************************************************************************
 *  Write user data, Page 0 and Page 4
 *
 ******************************************************************************/
    for(ii=0;ii<(USER_DATA_SIZE-2);ii++)
    {
        writeData(page0_I2C_addr,(uint8_t)userDataImage[ii].address,userDataImage[ii].data);
    }
    // Watchdog information
    writeData(page1Image[PAGE1_I2C2_OFFSET].data,0x05,userDataImage[ii++].data);
    writeData(page1Image[PAGE1_I2C2_OFFSET].data,0x09,userDataImage[ii].data);

/*******************************************************************************
 *  Write Non-NVM user data, Page 0 and Page 4
 *
 ******************************************************************************/
    for(ii=0;ii<(26);ii++)
    {
        if(ii != 3)
        {
            writeData(page0_I2C_addr,userDataNonNVM_data[ii][ADDRESS],userDataNonNVM_data[ii][DATA]);
        }
    }
    // Watchdog information
    writeData(page1Image[PAGE1_I2C2_OFFSET].data,0x01,userDataNonNVM_data[ii++][1]);
    writeData(page1Image[PAGE1_I2C2_OFFSET].data,0x03,userDataNonNVM_data[ii++][1]);
    writeData(page1Image[PAGE1_I2C2_OFFSET].data,0x04,userDataNonNVM_data[ii++][1]);
    writeData(page1Image[PAGE1_I2C2_OFFSET].data,0x07,userDataNonNVM_data[ii++][1]);

/*******************************************************************************
 *  Write Page 1 data
 *  This includes locking the Frequency selection.
 *
 ******************************************************************************/
    for(ii=0;ii<(PAGE1_DATA_SIZE);ii++)
    {
        writeData(page0_I2C_addr+1,(uint8_t)(page1Image[ii].address-0x100),page1Image[ii].data);
    }

/*******************************************************************************
 *  Write Page 3 data, sub page 0
 *
 ******************************************************************************/
    writeData(page0_I2C_addr,0xA4,0x00);
    for(ii=0;ii<(PAGE3_DATA_SIZE);ii++)
    {
        writeData(page0_I2C_addr+3,(uint8_t)ii,page3sub0Image[ii]);
    }

/*******************************************************************************
 *  Write Page 3 data, sub page 1
 *
 ******************************************************************************/
    writeData(page0_I2C_addr,0xA4,0x01);
    for(ii=0;ii<(PAGE3_DATA_SIZE);ii++)
    {
        writeData(page0_I2C_addr+3,(uint8_t)ii,page3sub1Image[ii]);
    }

/*******************************************************************************
 *  Write Page 3 data, sub page 2
 *
 ******************************************************************************/
    writeData(page0_I2C_addr,0xA4,0x02);
    for(ii=0;ii<(PAGE3_DATA_SIZE);ii++)
    {
        writeData(page0_I2C_addr+3,(uint8_t)ii,page3sub2Image[ii]);
    }

    // Return register to the default value.
    writeData(page0_I2C_addr,0xA4,0x00);

/*******************************************************************************
 *  Keep NVM unlocked
 *  Page 1, address 0x41, data = 0xA5.  Anything other than 0xA5 locks the NVM
 *
 ******************************************************************************/

    //
    writeData(page1Image[34].data+1,0x41,0xA5);

/******************************************************************************
 * Update Register CRC
 *
 *
 ******************************************************************************/

    /*
     * Check if register CRC is enabled.
     */
    if(page1Image[REG_CRC_EN_OFFSET].data & 0x40)
    {
        // Clear 0xF8, CRC_9
        writeData(page0_I2C_addr,0xF8,0x00);
        // Update Register CRC
        writeData(page0_I2C_addr,0xEF,0x02);
        // Wait for 0xF8, CRC_9 to be non-Zero
        tempByte = readData(page0_I2C_addr,0xF8);
        while(tempByte == 0)
        {
            tempByte = readData(page0_I2C_addr,0xF8);
        }
        tempByte = readData(page0_I2C_addr,userDataNonNVM_data[ENDRV_STAT_OFFSET][ADDRESS]);
        if(!(tempByte == userDataNonNVM_data[ENDRV_STAT_OFFSET][DATA]))
        {
            // Calculate new CRC for User Registers, page 0 and page 4
            includeCRC = 0xFFFF;
            excludeCRC = 0xFFFF;
            ii=0;
            tempByte = 0;
            // Register 0
            includeCRC = crcD8(includeCRC, tempByte & userDataCRC_mask[ii][MASK_INCLUDE]);
            excludeCRC = crcD8(excludeCRC, tempByte & userDataCRC_mask[ii][MASK_EXCLUDE]);
            ii++;
            while(ii < 4)
            {
                // Register 1,2,3
                tempByte = readData(page0_I2C_addr,ii);
                includeCRC = crcD8(includeCRC, tempByte & userDataCRC_mask[ii][MASK_INCLUDE]);
                excludeCRC = crcD8(excludeCRC, tempByte & userDataCRC_mask[ii][MASK_EXCLUDE]);
                ii++;
            }
            NonNVMIndex = 0;
            NVMIndex = 0;
            while(ii < 240)
            {
                tempByte = 0;
                if(userDataCRC_mask[ii][MASK_INCLUDE])
                {
                    if(ii == userDataImage[NVMIndex].address)
                    {
                        tempByte = userDataImage[NVMIndex].data;
                        NVMIndex++;
                    }
                    else if(ii == userDataNonNVM_data[NonNVMIndex][ADDRESS])
                    {
                        tempByte = userDataNonNVM_data[NonNVMIndex][DATA];
                        NonNVMIndex++;
                    }
                }
                includeCRC = crcD8(includeCRC, tempByte & userDataCRC_mask[ii][MASK_INCLUDE]);
                excludeCRC = crcD8(excludeCRC, tempByte & userDataCRC_mask[ii][MASK_EXCLUDE]);
                ii++;
            }
            /*
             * Contents in user register space 0x401-0x409 are placed in locations 0xF0-0xF8 for CRC calculation
             */
            while(ii < 256)
            {
                tempByte = 0;
                if(userDataCRC_mask[ii][MASK_INCLUDE])
                {
                    /*
                     *  0x401 - 0x311 = 0xF0
                     */
                    if(ii == (userDataImage[NVMIndex].address - 0x311))
                    {
                        tempByte = userDataImage[NVMIndex].data;
                        NVMIndex++;
                    }
                    else if(ii == userDataNonNVM_data[NonNVMIndex][ADDRESS])
                    {
                        tempByte = userDataNonNVM_data[NonNVMIndex][DATA];
                        NonNVMIndex++;
                    }
                }
                includeCRC = crcD8(includeCRC, tempByte & userDataCRC_mask[ii][MASK_INCLUDE]);
                excludeCRC = crcD8(excludeCRC, tempByte & userDataCRC_mask[ii][MASK_EXCLUDE]);
                ii++;
            }

            tempByte = (uint8_t)(0x00FF & includeCRC);
            writeData(page0_I2C_addr,0xF0,tempByte);
            tempByte = (uint8_t)((0xFF00 & includeCRC)>>8);
            writeData(page0_I2C_addr,0xF1,tempByte);

            tempByte = (uint8_t)(0x00FF & excludeCRC);
            writeData(page0_I2C_addr,0xF2,tempByte);
            tempByte = (uint8_t)((0xFF00 & excludeCRC)>>8);
            writeData(page0_I2C_addr,0xF3,tempByte);
        }
    }

/******************************************************************************
 * Burn EEPROM from SRAM
 *
 ******************************************************************************/

    //
    writeData(page1Image[34].data+1,0xEF,0x02);
    tempByte = readData(page1Image[34].data+1,0xF3);
    while(tempByte & BIT1)
    {
        tempByte = readData(page1Image[34].data+1,0xF3);
    }

    while (1);
}

/*
 * GPIO ISR
 */
void PORT1_IRQHandler(void)
{
    uint32_t status;

    status = MAP_GPIO_getEnabledInterruptStatus(GPIO_PORT_P1);
    MAP_GPIO_clearInterruptFlag(GPIO_PORT_P1, status);

    MAP_Interrupt_disableSleepOnIsrExit();

}

/*******************************************************************************
 * The USCIAB0TX_ISR is structured such that it can be used to transmit any
 * number of bytes by pre-loading TXByteCtr with the byte count. Also, TXData
 * points to the next byte to transmit.
 ******************************************************************************/
void EUSCIB0_IRQHandler(void)
{
    uint_fast16_t status;

    status = MAP_I2C_getEnabledInterruptStatus(EUSCI_B0_BASE);
    MAP_I2C_clearInterruptFlag(EUSCI_B0_BASE, status);

    if (status & EUSCI_B_I2C_STOP_INTERRUPT)
    {
        MAP_Interrupt_disableSleepOnIsrExit();
        MAP_I2C_disableInterrupt(EUSCI_B0_BASE,
                                 EUSCI_B_I2C_STOP_INTERRUPT);
    }

}
